/*
 * 26 April 2007
 * Operations on amino sequences of structure: struct seq
 * $Id: seq_i.h,v 1.1 2007/09/28 16:57:06 mmundry Exp $
 */

#ifndef SEC_I_H
#define SEC_I_H

struct seq *seq_duplicate (const struct seq *seq, size_t n);

#endif /* SEC_I_H */


