/* 30 Oct 2007
 * Accessor functions for compound information 
 */
#ifndef COMPOUND_H
#define COMPOUND_H


#include "prob_vec.h"
#include "coord.h"


char * get_compound_vec(struct prob_vec *v);
char * get_compound_coord(struct coord *c);

#endif  /* COORD_H */
