/*
 * 25 Nov 2005
 * There is a protocol for saying that a dihedral angle cannot be
 * calculated. Flag it with this value.
 * rcsid = $Id: bad_angle.h,v 1.1 2007/09/28 16:57:09 mmundry Exp $
 */
#ifndef BAD_ANGLE_H
#define BAD_ANGLE_H
extern const float BAD_ANGLE;                  /* some unlikely angle */
#endif /* BAD_ANGLE_H */
